#!/usr/bin/env Rscript
library(REddyProc)
library(dplyr)
library(lubridate)
library(tidyr)
fileName <- 'input.txt'
metaName <- 'metadata.txt'

                                        #Read in MetaData file which has 5 lines
                                        # First line: latitude, longitude, timezone offset from UTC
                                        # Second line: Name of ustar variable
                                        # Third line: Ameriflux variable names to gap fill with u* screening
                                        # Fourth line: REddyProc names that match those
                                        # Fifth line: Ameriflux variable names to gap fill without u* screen
                                        #Sixth line: REddyProc names that match those

#Note: Tair, Rg, and Ustar are required

metaData <- readLines(metaName)
metaLoc <- unlist(strsplit(metaData[1],","))
latitude <- as.double(metaLoc[1])
longitude <- as.double(metaLoc[2])
timezone <- as.integer(metaLoc[3])
ustarname <- unlist(metaData[2])
metaCol_ustar_AM <- unlist(strsplit(metaData[3],","))
metaCol_ustar <- unlist(strsplit(metaData[4],","))
ustarloop <- seq(length(metaCol_ustar))
metaCol_AM <- unlist(strsplit(metaData[5],","))
metaCol <- unlist(strsplit(metaData[6],","))
noustarloop <- seq(length(metaCol))

                                        #Read Ameriflux file and rename columns
AM_site <- read.csv(fileName, na.strings = c("-9999", -9999))
names(AM_site)[names(AM_site) == ustarname] <- 'Ustar'
for (uloc in ustarloop) { names(AM_site)[names(AM_site) == metaCol_ustar_AM[uloc]] <- metaCol_ustar[uloc] }
for (nuloc in noustarloop) { names(AM_site)[names(AM_site) == metaCol_AM[nuloc]] <- metaCol[nuloc] }
AM_site <- AM_site[, c("TIMESTAMP_START","TIMESTAMP_END","Ustar",metaCol_ustar,metaCol)]

                                        #Parse DatTime stamp TIMESTAMP_END to REddyproc standard 
AM_site$DateTime <- parse_date_time(AM_site$TIMESTAMP_END, "%Y%m%d%H%M", tz="UTC")
AM_site <- separate(AM_site, DateTime, into = c("Date","Time"), sep = " ", extra = "merge", remove = F)
AM_site <- separate(AM_site, Date, into = c("Year","Month","Day"), sep = "-", extra = "merge", remove = F)
AM_site$DoY <- as.integer(strftime(AM_site$Date, format = "%j"))
AM_site$Year<-sapply(AM_site$Year,as.integer)
AM_site <- AM_site %>%
  mutate(Hour =
           strsplit(Time, ":") %>%
           sapply(function(x){
             x <- as.numeric(x)
             x[1] + x[2]/60 + x[3]/(60*60)
           })
  )
AM_site$Rg <- ifelse(AM_site$Rg<0, 0, AM_site$Rg)

EddyData <- AM_site[, c("Year","DoY","Hour","Ustar",metaCol_ustar,metaCol)]

#Filter runs
i<-1
for (i in ustarloop) { EddyData <- filterLongRuns(EddyData,  metaCol_ustar[i]) }

#Convert time
EddyDataWithPosix <- fConvertTimeToPosix(EddyData, 'YDH',Year = 'Year',Day = 'DoY', Hour = 'Hour')

#Set up Reddyproc
DTS <- ifelse(EddyData$Hour[2]-EddyData$Hour[1]<1,48,24)
EProc <- sEddyProc$new('site', EddyDataWithPosix, c('Ustar',metaCol_ustar, metaCol), DTS=DTS)
EProc$sSetLocationInfo(LatDeg = latitude, LongDeg = longitude,TimeZoneHour = timezone)

#Estimate ustar cutoff
uStarTh <- EProc$sEstUstarThold()
EProc$sGetUstarScenarios()

#Gap fill met variables
i<-1
for (i in noustarloop) { EProc$sMDSGapFill(metaCol[i],FillAll = FALSE,  minNWarnRunLength = NA) }

#Gap fill flux variables with u* cutoff
i<-1
for (i in ustarloop) { EProc$sMDSGapFillUStarScens(metaCol_ustar[i] ,FillAll = FALSE,  minNWarnRunLength = NA) }

#Partition with nighttime
EProc$sMRFluxPartitionUStarScens('NEE_f')

                                        #EddyProc.C$sMRFluxPartition('NEE_f', parsE0Regression = list(TempRange = 3))	# night time partitioning of NEE without ustar filter -> Reco, GPP
#EddyProc.C$sMRFluxPartitionUStarScens('NEE_uStar_f')	# night time partitioning of NEE with ustar filter -> Reco, GPP
#EddyProc.C$sGLFluxPartition('NEE_uStar_f')	# day time partitioning after Lasslop et al. (2010) -> Reco_DT, GPP_DT
#EddyProc.C$sTKFluxPartition('NEE_uStar_f')	# day time partitioning after Lasslop 2015 -> Reco_DT, GPP_DT
#EddyProc.C$sTKFluxPartitionUStarScens('NEE_uStar_f')	# day time partitioning after Lasslop 2015 and ustar filter -> Reco_DT, GPP_DT
#Output data

FilledEddyData <- EProc$sExportResults()
CombinedData <- cbind(EddyData, FilledEddyData)
fWriteDataframeToFile(CombinedData, 'output.txt')
